
Require Import List.
Require Import ZArith.
Open Scope list_scope.

Section F_matrix.
  
  Variable F : Type.
  Variable add sub mul : F->F->F.
  Notation "a + b" := (add a b).
  Notation "a - b" := (sub a b).
  Notation "a * b" := (mul a b).

  Axiom comm_add : forall (a b : F), a + b = b + a.
  Axiom assoc_add : forall (a b c : F), a + (b + c) = (a + b) + c.
  Axiom comm_mul : forall (a b : F), a * b = b * a.
  Axiom assoc_mul : forall (a b c : F), a * (b * c) = (a * b) * c. 

  Variable zero one : F.
  Axiom zero_add_left : forall (c : F), zero + c = c.
  Lemma zero_add_right : forall (c : F), c + zero = c.
  Proof.
    intros.
    rewrite comm_add.
    rewrite zero_add_left.
    reflexivity.
  Qed.
  
  Axiom one_mul_left : forall (c : F), one * c = c.
  Lemma one_mul_right : forall (c : F), c * one = c.
  Admitted.
  
  Axiom zero_mul_left : forall (c : F), zero * c = zero.
  Lemma zero_mul_right : forall (c : F), c * zero = zero.
  Admitted.


  Axiom distrib_left : forall (a b c : F), a * (b + c) = a * b + a * c.
  Lemma distrib_right : forall (a b c : F), (a + b) * c = a * c + b * c.
  Proof.
    intros.
    rewrite comm_mul.
    rewrite distrib_left.
    rewrite comm_mul.
    rewrite comm_add.
    rewrite comm_mul.
    rewrite comm_add.
    reflexivity.
  Qed.

  Lemma distrib_mutual : forall (a b c d : F), (a + b) * (c + d) = (a * c + b * c) + (a * d + b * d).
  Proof.
    intros.
    rewrite distrib_left.
    rewrite distrib_right.
    assert ((a + b) * d = a * d + b * d) as H.
    - rewrite distrib_right.
      reflexivity.
    - rewrite H.
      reflexivity.
  Qed.

  Fixpoint init_row_list (n : nat) (f : nat -> F) : list F :=
    match n with
      | O => nil
      | S n' => app (init_row_list n' f) ((f n') :: nil)
    end.

  Fixpoint init_matrix_list (m n : nat) (f : nat -> nat -> F) : list F :=
    match m with
      | O => nil
      | S m' => app (init_matrix_list m' n f) (init_row_list n (f m'))
    end.

  Compute init_matrix_list 2 2 (fun n m => one).

  Definition elem (n i j : nat) (ul : list F) : F := nth (plus (mult n i) j) ul zero.
  Variables x11 x12 x21 x22 y11 y12 y21 y22 : F.
  Compute elem 2 1 0 (x11::x12::x21::x22::nil).

  (* 行列の和 *)
  Definition add_matrix_list (m n : nat) (u v : list F) : list F := init_matrix_list m n (fun i j => (elem m i j u) + (elem m i j v)). 
  Compute add_matrix_list 2 2 (x11::x12::x21::x22::nil) (y11::y12::y21::y22::nil).
  
  Fixpoint bilinear_list (m i : nat) (ul : list F) (n j : nat) (vl : list F) (k : nat) : F :=
    match k with
      | O => zero
      | S k' => (bilinear_list m i ul n j vl k') + (elem m i k' ul) * (elem n k' j vl)
    end.

  (* 計算例 *)
  Compute bilinear_list 2 0 (x11::x12::x21::x22::nil) 2 0 (y11::y12::y21::y22::nil) 2.
  Compute bilinear_list 2 1 (x11::x12::x21::x22::nil) 1 0 (y11::y21::nil) 2.
  Compute bilinear_list 2 0 (x11::x12::x21::x22::nil) 1 0 (y11::y21::nil) 2.

  (* 行列の積 *)
  Definition mul_matrix_list (m1 n1 : nat) (ul : list F) (m2 n2 : nat) (vl : list F) : list F :=
    init_matrix_list m1 n2 (fun i j => bilinear_list n1 i ul n2 j vl m2).

  Compute mul_matrix_list 2 2 (x11::x12::x21::x22::nil) 2 2 (y11::y12::y21::y22::nil).
  Variables a11 a12 a13 a21 a22 a23 a31 a32 a33 b11 b12 b13 b21 b22 b23 b31 b32 b33 : F.
  
  (* 転置行列 *)
  Definition transpose (m n : nat) (u : list F) : list F :=
    init_matrix_list m n (fun i j => elem m j i u).


  (*   *)
  Fixpoint beq_nat (n m : nat) : bool :=
  match n with
  | O => match m with
         | O => true
         | S m' => false
         end
  | S n' => match m with
            | O => false
            | S m' => beq_nat n' m'
            end
  end.

  (* *)
  Fixpoint ble_nat (n m : nat) : bool :=
    match n with
      | O => true
      | S n' =>
        match m with
          | O => false
          | S m' => ble_nat n' m'
        end
    end.
  
  Definition blt_nat (n m : nat) : bool :=
    if andb (ble_nat n m) (negb (beq_nat n m)) then true else false.

  Compute blt_nat 1 2.
  Compute blt_nat 2 2.
  
  (* 余因子行列 *)
  Definition cofactor_elem (n i' j' : nat) (ul : list F) (i j : nat) : F :=
    match blt_nat i i', blt_nat j j' with
      | true, true => elem n i j ul
      | true, false => elem n i (j+1) ul
      | false, true => elem n (i+1) j ul
      | false, false => elem n (i+1) (j+1) ul
    end.

  Definition mat33 := a11::a12::a13::a21::a22::a23::a31::a32::a33::nil.
  Compute cofactor_elem 3 0 0 mat33 0 0.
  Compute cofactor_elem 3 0 0 mat33 0 1.
  Compute cofactor_elem 3 0 0 mat33 1 0.
  Compute cofactor_elem 3 0 0 mat33 1 1.

  Compute cofactor_elem 3 1 1 mat33 0 0.
  Compute cofactor_elem 3 1 1 mat33 0 1.
  Compute cofactor_elem 3 1 1 mat33 1 0.
  Compute cofactor_elem 3 1 1 mat33 1 1.
  
  Definition cofactor_matrix (m n i j : nat) (u : list F) : list F :=
    init_matrix_list (m-1) (n-1) (cofactor_elem m i j u).
  
  Compute cofactor_matrix 3 3 0 0 mat33.
  Compute cofactor_matrix 3 3 1 1 mat33.
  Compute cofactor_matrix 3 3 2 2 mat33.

  Fixpoint mod2 n : nat :=
    match n with
      | O => O
      | S O => S O
      | S (S n') => mod2 n'
    end.

  Compute mod2 2.
  Compute mod2 5.
  Compute mod2 10.
  
  Fixpoint list_sign_sum_F (n : nat) (ret : F) (l : list F) : F :=
    match n with
      | O => ret
      | S n' => match mod2 n' with
                      | O => list_sign_sum_F n' (ret + nth n' l zero) l
                      | S O => list_sign_sum_F n' (ret - nth n' l zero) l
                      | _ => list_sign_sum_F n' ret l
                end
    end.

  Compute list_sign_sum_F 3 zero (a11::a12::a13::nil).
  Compute list_sign_sum_F 2 zero (a11::a12::nil).

  (* 行列式 *)
  Fixpoint determinant (n : nat) (u : list F) : F :=
    match n with
      | O => zero
      | 1 => elem 1 0 0 u
      | S n' =>
        list_sign_sum_F n zero (init_row_list n (fun k => (elem n 0 k u) * (determinant n' (cofactor_matrix n n 0 k u))))
    end.

  Compute determinant 2 (a11::a12::a21::a22::nil).
  Compute determinant 3 (a11::a12::a13::a21::a22::a23::a31::a32::a33::nil).
  
End F_matrix.

Require Import ExtrOcamlNatInt.
Extraction "matrix.ml" init_matrix_list add_matrix_list mul_matrix_list cofactor_matrix determinant.