
Section Vector3d.

  Variable F : Type.
  Variables add mul : F->F->F.
  Variables neg inv : F->F.

  
  Notation "a + b" := (add a b).
  Notation "a * b" := (mul a b).

  Variables one zero : F.

  Axiom comm_add : forall (a b : F), a + b = b + a.
  Axiom assoc_add : forall (a b c : F), a + (b + c) = (a + b) + c.
  Axiom comm_mul : forall (a b : F), a * b = b * a.
  Axiom assoc_mul : forall (a b c : F), a * (b * c) = (a * b) * c. 

  Axiom zero_add_left : forall (c : F), zero + c = c.
  Lemma zero_add_right : forall (c : F), c + zero = c.
  Admitted.

  Axiom neg_add_left : forall (c : F), neg c + c = zero.
  Lemma neg_add_right : forall (c : F), c + neg c = zero.
  Admitted.
  
  Axiom one_mul_left : forall (c : F), one * c = c.
  Lemma one_mul_right : forall (c : F), c * one = c.
  Admitted.
  
  Axiom zero_mul_left : forall (c : F), zero * c = zero.
  Lemma zero_mul_right : forall (c : F), c * zero = zero.
  Admitted.

  
  Axiom distrib_left : forall (a b c : F), a * (b + c) = a * b + a * c.
  Lemma distrib_right : forall (a b c : F), (a + b) * c = a * c + b * c.
  Admitted.
    
  Lemma eq_add : forall (a b c : F), a = b -> a + c = b + c.
  Proof.
    intros.
    rewrite H.
    reflexivity.
  Qed.

  Lemma eq_sub : forall (a b c : F), a = b -> a + neg c = b + neg c.
  Proof.
    intros.
    apply eq_add.
    exact H.
  Qed.

  Lemma zero_mul : forall (a : F), a * zero = zero.
  Proof.
    intros.
    assert (a * (zero + zero) = a * zero).
    rewrite zero_add_left.
    reflexivity.
    rewrite distrib_left in H.
    assert (a*zero + a*zero + neg (a*zero) = a*zero + neg (a*zero)).
    apply eq_sub.
    exact H.
    rewrite <- assoc_add in H0.
    rewrite neg_add_right in H0.
    rewrite zero_add_right in H0.
    exact H0.
  Qed.
    
  Lemma eq_mul : forall (a b c : F), a = b -> a * c = b * c.
  Proof.
    intros.
    rewrite H.
    reflexivity.
  Qed.

  Lemma double_neg_add_left : forall (a : F), a + neg a + neg (neg a) = neg (neg a).
  Proof.
    intros.
    rewrite neg_add_right.
    rewrite comm_add.
    rewrite zero_add_right.
    reflexivity.
  Qed.

  Lemma double_neg_add_right : forall (a : F), a + neg a + neg (neg a) = a.
  Proof.
    intros.
    rewrite <- assoc_add.
    rewrite neg_add_right.
    rewrite zero_add_right.
    reflexivity.
  Qed.
    
  Lemma double_neg : forall (a : F), a = neg (neg a). 
  Proof.
    intros.
    rewrite <- double_neg_add_left.
    rewrite double_neg_add_right.    
    reflexivity.
  Qed.

  Lemma neg_one_mul_right : forall (a : F), a * (neg one) = neg a.
  Proof.
    intros.
    assert (a * (one + neg one) = zero) as Hneg.
    rewrite neg_add_right.
    rewrite zero_mul.
    reflexivity.
    rewrite distrib_left in Hneg.
    rewrite one_mul_right in Hneg.
    assert (a * neg one + a + neg a = zero + neg a) as Hzn.
    rewrite comm_add in Hneg.
    apply eq_sub.
    exact Hneg.
    rewrite <- assoc_add in Hzn.
    rewrite neg_add_right in Hzn.
    rewrite zero_add_right in Hzn.
    rewrite comm_add in Hzn.
    rewrite zero_add_right in Hzn.
    exact Hzn.
  Qed.

  Lemma neg_one_mul_left : forall (a : F), neg one * a = neg a.
  Proof.
    intros.
    rewrite comm_mul.
    rewrite neg_one_mul_right.
    reflexivity.
  Qed.
    
  Lemma neg_square : neg one * neg one = one.
  Proof.
    assert (neg one * neg one = neg (neg one)).
    rewrite neg_one_mul_left.
    reflexivity.
    rewrite <- double_neg in H.
    exact H.
  Qed.

  Lemma eq_add_each : forall (a b c d : F), a = b -> c = d -> a + c = b + d.
  Proof.
    intros.
    rewrite H; rewrite H0; reflexivity.
  Qed.

  Lemma eq_mul_each : forall (a b c d : F), a = b -> c = d -> a * c = b * d.
  Proof.
    intros.
    rewrite H; rewrite H0; reflexivity.
  Qed.
  
  Record vector3d := { x : F; y : F; z : F }.

  (* 加法 *)
  Definition addv3 (v1 v2 : vector3d) : vector3d :=
    Build_vector3d ((x v1) + (x v2)) ((y v1) + (y v2)) ((z v1) + (z v2)).

  (* 実数倍 *)
  Definition fmulv3 (c : F) (v : vector3d) : vector3d :=
    Build_vector3d (c * (x v)) (c * (y v)) (c * (z v)).

  (* 内積 *)
  Definition in_prod3 (v1 v2 : vector3d) : F :=
    (x v1)*(x v2) + (y v1)*(y v2) + (z v1)*(z v2).

  (* 外積 *)
  Definition ext_prod3 (v1 v2 : vector3d) : vector3d :=
    let vr_x := (y v1) * (z v2) + neg ((y v2) * (z v1)) in
    let vr_y := (z v1) * (x v2) + neg ((z v2) * (x v1)) in
    let vr_z := (x v1) * (y v2) + neg ((x v2) * (y v1)) in
    Build_vector3d vr_x vr_y vr_z.

  Variables x1 y1 z1 x2 y2 z2 x3 y3 z3: F.
  Compute ext_prod3 (Build_vector3d x1 y1 z1) (Build_vector3d x2 y2 z2).
  
  (* 外積の反交換性 *)
  Lemma skew_comm : addv3 (ext_prod3 (Build_vector3d x1 y1 z1) (Build_vector3d x2 y2 z2)) (ext_prod3 (Build_vector3d x2 y2 z2) (Build_vector3d x1 y1 z1)) = Build_vector3d zero zero zero.
  Proof.
    compute.
    
    assert (neg (y2 * z1) + y2 * z1 = zero) as Hx.
    rewrite neg_add_left.
    reflexivity.
    
    assert (neg (y2 * z1) + (y2 * z1 + neg (y1 * z2)) = neg (y1*z2)) as Hx'.
    rewrite assoc_add.
    rewrite Hx.
    rewrite zero_add_left.
    reflexivity.
    
    assert (neg (z2 * x1) + z2 * x1 = zero) as Hy.
    rewrite neg_add_left.
    reflexivity.
    assert (neg (z2 * x1) + (z2 * x1 + neg (z1 * x2)) = neg (z1*x2)) as Hy'.
    rewrite assoc_add.
    rewrite Hy.
    rewrite zero_add_left.
    reflexivity.
    
    assert (neg (x2 * y1) + x2 * y1 = zero) as Hz.
    rewrite neg_add_left.
    reflexivity.
    assert (neg (x2 * y1) + (x2 * y1 + neg (x1 * y2)) = neg (x1*y2)) as Hz'.
    rewrite assoc_add.
    rewrite Hz.
    rewrite zero_add_left.
    reflexivity.

    repeat rewrite <- assoc_add.
    rewrite Hx'; rewrite Hy'; rewrite Hz'.
    repeat rewrite neg_add_right.
    reflexivity.
  Qed.

  (* Lie algebra としての扱い *)
  (* 基底 *)
  Definition e1 := Build_vector3d one zero zero.
  Definition e2 := Build_vector3d zero one zero.
  Definition e3 := Build_vector3d zero zero one.
  Definition zero3 := Build_vector3d zero zero zero.

  Lemma mul_one_one : one * one = one.
  Admitted.
    
  Lemma mul_zero_zero : zero * zero = zero.
  Admitted.
  
  Lemma mul_one_zero : one * zero = zero.
  Admitted.

  Lemma mul_zero_one : zero * one = zero.
  Admitted.

  Lemma add_zero_zero : zero + zero = zero.
  Admitted.

  Lemma add_zero_one : zero + one = one.
  Admitted.

  Lemma add_one_zero : one + zero = one.
  Admitted.

  Lemma neg_zero : neg zero = zero.
  Admitted.
  
  (* *)
  Lemma ext_prod3_relation_e1e2 : ext_prod3 e1 e2 = e3.
  Proof.
    compute.
    repeat rewrite mul_zero_zero.
    rewrite mul_one_zero.
    rewrite mul_zero_one.
    rewrite mul_one_one.
    repeat rewrite neg_zero.
    repeat rewrite add_zero_zero.
    rewrite add_one_zero.
    reflexivity.
  Qed.

  Lemma ext_prod3_relation_e2e3 : ext_prod3 e2 e3 = e1.
  Proof.
    compute.
    repeat rewrite mul_zero_zero.
    rewrite mul_one_zero.
    rewrite mul_zero_one.
    rewrite mul_one_one.
    repeat rewrite neg_zero.
    repeat rewrite add_zero_zero.
    rewrite add_one_zero.
    reflexivity.
  Qed.

  Lemma ext_prod3_relation_e3e1 : ext_prod3 e3 e1 = e2.
  Proof.
    compute.
    repeat rewrite mul_zero_zero.
    rewrite mul_one_zero.
    rewrite mul_zero_one.
    rewrite mul_one_one.
    repeat rewrite neg_zero.
    repeat rewrite add_zero_zero.
    rewrite add_one_zero.
    reflexivity.
  Qed.

  Variables a1 a2 a3 b1 b2 b3 c1 c2 c3: F.
  Compute ext_prod3 (Build_vector3d a1 a2 a3) (Build_vector3d b1 b2 b3).
  Compute ext_prod3 (Build_vector3d b1 b2 b3) (Build_vector3d c1 c2 c3).
  Compute ext_prod3 (Build_vector3d a1 a2 a3) (Build_vector3d (b2 * c3 + neg (c2 * b3)) (b3 * c1 + neg (c3 * b1)) (b1 * c2 + neg (c1 * b2))).
  Compute ext_prod3 (Build_vector3d a1 a2 a3) (ext_prod3 (Build_vector3d b1 b2 b3) (Build_vector3d c1 c2 c3)).
  
  Compute in_prod3 (Build_vector3d a1 a2 a3) (Build_vector3d c1 c2 c3).
  Compute in_prod3 (Build_vector3d a1 a2 a3) (Build_vector3d b1 b2 b2).

  (* 以下は証明できるが人間が見た方が速い条件 *)
  Parameter triple_product_lhs_x : a2 * (b1 * c2 + neg (c1 * b2)) + neg (b3 * c1 + neg (c3 * b1)) * a3 = a2 * b1 * c2 + neg (a2 * b2 * c1) + neg (a3 * b3 * c1) + a3 * b1 * c3.
  Parameter triple_product_rhs_x : (a1 * c1 + a2 * c2 + a3 * c3) * b1 + (a1 * b1 + a2 * b2 + a3 * b3) * neg c1 = a2 * b1 * c2 + neg (a2 * b2 * c1) + neg (a3 * b3 * c1) + a3 * b1 * c3.

  (* ベクトル 3 重積の x 成分 *)
  Lemma tripule_x : a2 * (b1 * c2 + neg (c1 * b2)) + neg ((b3 * c1 + neg (c3 * b1))) * a3 = (a1 * c1 + a2 * c2 + a3 * c3) * b1 + (a1 * b1 + a2 * b2 + a3 * b3) * neg c1.
  Proof.
    intros.
    remember triple_product_lhs_x as Hlhs_x. 
    rewrite Hlhs_x.
    remember triple_product_rhs_x as Hrhs_x.
    rewrite Hrhs_x.
    reflexivity.
  Qed.

End Vector3d.